"""Main AfterShip object."""

from typing import Optional

from aiohttp import ClientSession

from pyaftership.couriers import AfterShipCouriers

from .base import AfterShipBase
from .trackings import AfterShipTrackings


class AfterShip(AfterShipBase):
    """Main AfterShip object."""

    def __init__(self, api_key: str, session: ClientSession, timeout: int = 10) -> None:
        """Initialize."""
        AfterShipBase._api_key = api_key
        AfterShipBase._session = session
        AfterShipBase._timeout = timeout

        self.trackings = AfterShipTrackings()
        self.couriers = AfterShipCouriers()



"""Custom exceptions."""


class AfterShipException(Exception):
    """Base exception for AfterShip."""


class AfterShipCommunicationException(AfterShipException):
    """Communication exception for AfterShip."""



"""AfterShip Trackings object."""
from .base import AfterShipBase


class AfterShipTrackings(AfterShipBase):
    """AfterShip Trackings object."""

    async def list(self) -> dict:
        """
        List tracking information.

        https://docs.aftership.com/api/4/trackings/get-trackings
        """
        return await self._call_api("trackings")

    async def add(
        self,
        tracking_number: str,
        title: str = None,
        slug: str = None,
        tracking_postal_code: any = None,
    ) -> dict:
        """
        Add tracking information.

        https://docs.aftership.com/api/4/trackings/post-trackings
        """

        data = {"tracking": {"tracking_number": tracking_number}}

        if slug is not None:
            data["tracking"]["slug"] = slug

        if title is not None:
            data["tracking"]["title"] = title

        if tracking_postal_code is not None:
            data["tracking"]["tracking_postal_code"] = tracking_postal_code

        return await self._call_api("trackings", method="POST", data=data)

    async def remove(self, tracking_number: str, slug: str) -> dict:
        """
        Add tracking information.

        https://docs.aftership.com/api/4/trackings/delete-trackings
        """
        return await self._call_api(
            f"trackings/{slug}/{tracking_number}",
            method="DELETE",
            data={"tracking": {"tracking_number": tracking_number}},
        )



"""AfterShip Base."""
import asyncio
from socket import gaierror
from typing import TYPE_CHECKING, Optional

from aiohttp import ClientError, ClientResponse, ClientSession, ClientTimeout

from .const import BASE_URL, GOOD_HTTP_CODES
from .exceptions import AfterShipCommunicationException

if TYPE_CHECKING:
    from .trackings import AfterShipTrackings


class AfterShipBase:
    """Aftership Base object."""

    _api_key: str
    _session: ClientSession
    _timeout: int
    _trackings: Optional["AfterShipTrackings"]

    async def _call_api(
        self, endpoint: str, method: str = "GET", data: Optional[dict] = None
    ) -> Optional[ClientResponse]:
        """Private method to call the AfterShip API."""
        try:
            response = await self._session.request(
                method=method,
                url=f"{BASE_URL}/{endpoint}",
                headers={
                    "aftership-api-key": self._api_key,
                    "Content-Type": "application/json",
                },
                json=data,
                timeout=ClientTimeout(total=self._timeout),
            )
            return await self._handle_response(response)

        except asyncio.TimeoutError as exception:
            raise AfterShipCommunicationException("Timeout error") from exception

        except (ClientError, gaierror) as exception:
            raise AfterShipCommunicationException(
                f"Communication error {exception}"
            ) from exception

    async def _handle_response(self, response: ClientResponse) -> Optional[dict]:
        """Private method handle the result from AfterShip."""
        result = await response.json() if response else {}

        if response.status not in GOOD_HTTP_CODES:
            meta = result.get("meta", {})
            raise AfterShipCommunicationException(
                f"{response.status} is not valid - {meta.get('code')} - {meta.get('message')}"
            )

        return result.get("data", {})



"""Setup configuration."""
from setuptools import find_packages, setup

with open("README.md") as readme_file:
    readme = readme_file.read()

setup(
    author="Ludeeus",
    author_email="hi@ludeeus.dev",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    description="Async Python wrapper for the AfterShip API.",
    install_requires=["aiohttp>=3.6.1,<4.0"],
    license="MIT license",
    long_description_content_type="text/markdown",
    long_description=readme,
    name="pyaftership",
    packages=find_packages(include=["pyaftership", "pyaftership.*"]),
    url="https://github.com/ludeeus/pyaftership",
    version="23.1.0",
)



"""AfterShip Couriers object."""
from .base import AfterShipBase


class AfterShipCouriers(AfterShipBase):
    """AfterShip Couriers object."""

    async def detect(self, tracking_number: str, **kwargs) -> dict:
        """
        Detect couriers for tracking number.

        https://docs.aftership.com/api/4/couriers/post-couriers-detect
        """
        data = {"tracking": kwargs}
        data["tracking"]["tracking_number"] = tracking_number

        return await self._call_api("couriers/detect", method="POST", data=data)
